package eu.mhsl.craftattack.worldmuseum;

import eu.mhsl.craftattack.worldmuseum.commands.GamemodeCommand;
import eu.mhsl.craftattack.worldmuseum.commands.SpawnCommand;
import eu.mhsl.craftattack.worldmuseum.commands.TeleportCommand;
import eu.mhsl.craftattack.worldmuseum.listener.*;
import eu.mhsl.craftattack.worldmuseum.skins.SkinCache;
import eu.mhsl.craftattack.worldmuseum.util.TablistUpdateTask;
import eu.mhsl.craftattack.worldmuseum.util.Config;
import eu.mhsl.craftattack.worldmuseum.util.MuseumPlayer;
import eu.mhsl.craftattack.worldmuseum.handler.SignHandler;
import eu.mhsl.craftattack.worldmuseum.worlds.VoidWorld;
import eu.mhsl.craftattack.worldmuseum.worlds.WorldManager;
import net.minestom.server.MinecraftServer;
import net.minestom.server.entity.GameMode;
import net.minestom.server.event.GlobalEventHandler;
import net.minestom.server.event.inventory.InventoryPreClickEvent;
import net.minestom.server.event.item.ItemDropEvent;
import net.minestom.server.event.player.*;
import net.minestom.server.extras.bungee.BungeeCordProxy;
import net.minestom.server.timer.TaskSchedule;

import java.io.IOException;

public class Main {
    static int PORT = 25565;
    static final String IP = "0.0.0.0";

    public static void main(String[] args) throws IOException {
        // Initialization
        System.setProperty("minestom.chunk-view-distance", "16");
        MinecraftServer minecraftServer = MinecraftServer.init();
        MinecraftServer.getConnectionManager().setPlayerProvider(MuseumPlayer::new);
        MinecraftServer.setBrandName("mhsl.eu:worldmuseum_by_olischma");

        //update Taskbar
        MinecraftServer.getSchedulerManager().scheduleTask(new TablistUpdateTask(), TaskSchedule.tick(20), TaskSchedule.tick(20));

        //load eu.mhsl.craftattack.worldmuseum.worlds / configExample file
        WorldManager worldManager = WorldManager.getInstance();
        worldManager.loadWorlds();


        //load main config
        Config config = Config.getInstance();
        config.loadConfig();
        if (config.isBungeecordEnabled()) {
            BungeeCordProxy.enable();
            System.out.println("[Info] Bungeecord enabled");
        }


        GlobalEventHandler globalEventHandler = MinecraftServer.getGlobalEventHandler();

        //listeners
        globalEventHandler.addListener(PlayerUseItemEvent.class, new ItemUseListener());
        globalEventHandler.addListener(PlayerChunkUnloadEvent.class, new ChunkUnloading());
        globalEventHandler.addListener(InventoryPreClickEvent.class, new InventoryClickListener());
        globalEventHandler.addListener(PlayerBlockBreakEvent.class, new BlockBreakListener());
        globalEventHandler.addListener(PlayerBlockPlaceEvent.class, new BlockPlaceListener());
        globalEventHandler.addListener(PlayerDisconnectEvent.class, new DisconnectListener());

        //Handlers
        MinecraftServer.getBlockManager().registerHandler("minecraft:sign", SignHandler::new);

        //canceled Events
        globalEventHandler.addListener(PlayerSwapItemEvent.class, new EventCancel<>());
        globalEventHandler.addListener(ItemDropEvent.class, new EventCancel<>());

        //commands
        MinecraftServer.getCommandManager().register(new TeleportCommand());
        MinecraftServer.getCommandManager().register(new GamemodeCommand());
        MinecraftServer.getCommandManager().register(new SpawnCommand());

        globalEventHandler.addListener(PlayerLoginEvent.class, event -> {
            final MuseumPlayer player = (MuseumPlayer) event.getPlayer();
            final VoidWorld voidWorld = new VoidWorld();

            event.setSpawningInstance(voidWorld);
            MinecraftServer.getSchedulerManager().scheduleNextTick(() -> {
                player.setPermissionLevel(4);
                player.setRespawnPoint(voidWorld.getSpawn());
                player.setGameMode(GameMode.SPECTATOR);
                player.setAllowFlying(true);
                SkinCache.setSkin(player);
                voidWorld.movePlayer(player);
            });

            System.out.println("[Join] Player " + player.getUsername() +" joined the server.");
        });

        try {
            PORT = Integer.parseInt(args[0]);
        } catch (Exception e) {
            if (args.length != 0) System.out.println("Given port doesn't work.");
        }

        // Start the server on port default port 25565 if none is given
        System.out.println("[Info] Running on " + IP + ":" + PORT);
        minecraftServer.start(IP, PORT);
    }
}