package eu.mhsl.craftattack.worldmuseum.inventory;

import eu.mhsl.craftattack.worldmuseum.util.BunggeCordComunicator;
import eu.mhsl.craftattack.worldmuseum.util.Config;
import eu.mhsl.craftattack.worldmuseum.util.MuseumPlayer;
import eu.mhsl.craftattack.worldmuseum.worlds.World;
import eu.mhsl.craftattack.worldmuseum.worlds.WorldManager;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.minestom.server.MinecraftServer;
import net.minestom.server.inventory.Inventory;
import net.minestom.server.inventory.InventoryType;
import net.minestom.server.item.ItemStack;
import net.minestom.server.item.Material;
import net.minestom.server.timer.TaskSchedule;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class WorldSelector extends Inventory {
    public WorldSelector() {
        super(InventoryType.CHEST_1_ROW, Component.text("Weltenauswahl").color(NamedTextColor.DARK_GRAY));

        // Limit to seven worlds, so no overflow occurs // TODO make inventory bigger when more items are added
        List<World> worlds = WorldManager
                .getInstance()
                .getWorlds()
                .stream()
                .limit(7)
                .toList();

        Map<Integer, World> worldIndex = IntStream
                .range(0, worlds.size())
                .boxed()
                .collect(Collectors.toMap(i -> i, worlds::get));

        worldIndex.forEach((integer, world) -> setItemStack(integer, world.getItem()));

        setItemStack(8, ItemStack.of(Material.IRON_DOOR).withDisplayName(Component.text("Zurück zum Server").color(NamedTextColor.GOLD)));

        addInventoryCondition((p, slot, clickType, inventoryConditionResult) -> {
            MuseumPlayer player = (MuseumPlayer) p;
            inventoryConditionResult.setCancel(true);
            if(player.isLoading()) return;

            // leave World-Museum
            if(slot == 8) {
                player.closeInventory();
                player.startLoading();

                BunggeCordComunicator.connect(player, Config.getInstance().getBungeeReturnServerName());

                MinecraftServer.getSchedulerManager().scheduleTask(() -> {
                    if(player.isOnline()) player.kick("Timeout beim Serverwechsel. Bitte joine dem Server erneut!");
                }, TaskSchedule.seconds(5), TaskSchedule.stop());
                return;
            }

            // join any world
            if(worldIndex.containsKey(slot)) {
                player.closeInventory();
                worldIndex.get(slot).movePlayer(player);
            }
        });
    }
}
