package eu.mhsl.craftattack.worldmuseum.util;

import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.title.Title;
import net.minestom.server.MinecraftServer;
import net.minestom.server.entity.Player;
import net.minestom.server.event.player.PlayerMoveEvent;
import net.minestom.server.network.player.PlayerConnection;
import net.minestom.server.potion.Potion;
import net.minestom.server.potion.PotionEffect;
import net.minestom.server.timer.ExecutionType;
import net.minestom.server.timer.TaskSchedule;
import net.minestom.server.utils.MathUtils;
import org.jetbrains.annotations.NotNull;

import java.time.Duration;
import java.util.UUID;

public class MuseumPlayer extends Player {
    private boolean isLoading = false;

    public MuseumPlayer(@NotNull UUID uuid, @NotNull String username, @NotNull PlayerConnection playerConnection) {
        super(uuid, username, playerConnection);

        eventNode().addListener(PlayerMoveEvent.class, playerMoveEvent -> playerMoveEvent.setCancelled(isLoading));
    }

    public void startLoading() {
        isLoading = true;
        MinecraftServer.getSchedulerManager().submitTask(() -> {
            addEffect(new Potion(PotionEffect.BLINDNESS, (byte) 3, 600));
            showTitle(Title.title(
                    Component.text("Welt wird geladen").color(NamedTextColor.GOLD),
                    Component.text("...").color(NamedTextColor.GOLD),
                    Title.Times.times(Duration.ZERO, Duration.ofSeconds(30), Duration.ZERO)
            ));

            if(!isLoading) {
                clearEffects();
                clearTitle();
                return TaskSchedule.stop();
            }
            return TaskSchedule.millis(100);
        }, ExecutionType.ASYNC);
    }

    public void stopLoading() {
        isLoading = false;
    }

    public boolean isLoading() {
        return isLoading;
    }

    public void updateFlyingSpeed(float speed) {
        speed = MathUtils.clamp(getFlyingSpeed() + speed, 0.01f, 0.2f);
        setFlyingSpeed(speed);
        sendActionBar(
                Component
                        .text("Geschwindigkeit: ")
                        .color(NamedTextColor.GOLD)
                        .append(Component.text(Math.round(speed * 1000) + "%").color(NamedTextColor.WHITE))
        );
    }


}
