package eu.mhsl.craftattack.worldmuseum.worlds;

import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import eu.mhsl.craftattack.worldmuseum.items.Items;
import eu.mhsl.craftattack.worldmuseum.util.MuseumPlayer;
import net.kyori.adventure.text.Component;
import net.minestom.server.MinecraftServer;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.entity.GameMode;
import net.minestom.server.entity.Player;
import net.minestom.server.instance.AnvilLoader;
import net.minestom.server.instance.InstanceContainer;
import net.minestom.server.item.ItemStack;
import net.minestom.server.item.Material;
import net.minestom.server.utils.NamespaceID;
import net.minestom.server.world.DimensionType;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class World extends InstanceContainer implements PlayerMovable {
    private final static DimensionType dimension = DimensionType.builder(NamespaceID.from("mhsl.eu:worldview_by_olischma")).height(400).ambientLight(2.0f).build();

    private final boolean enabled;
    private final String name;
    private final Material material;
    private final Pos spawn;
    private final List<String> tags = new ArrayList<>();


    static {
        MinecraftServer.getDimensionTypeManager().addDimension(dimension);
    }

    public World(File file) throws IOException {
        super(UUID.randomUUID(), dimension, new AnvilLoader(file.getPath()));
        MinecraftServer.getInstanceManager().registerInstance(this);

        File configFile = new File(file.getPath() + "/config.json");
        if (configFile.createNewFile()) {
            writeDefaultConfig(file);
        }

        JsonObject config = (JsonObject) JsonParser.parseReader(new FileReader(file.getPath() + "/config.json"));

        this.enabled = config.get("enabled").getAsBoolean();
        this.name = config.get("name").getAsString();
        this.material = Material.fromNamespaceId("minecraft:" + config.get("material").getAsString().toLowerCase());

        JsonObject spawn = config.getAsJsonObject("spawn");
        this.spawn = new Pos(spawn.get("x").getAsDouble(), spawn.get("y").getAsDouble(), spawn.get("z").getAsDouble());

        JsonArray tags = config.getAsJsonArray("tags");
        tags.forEach(jsonElement -> this.tags.add(jsonElement.getAsString()));

    }

    @Override
    public void movePlayer(Player player) {
        MuseumPlayer p = (MuseumPlayer) player;

        p.startLoading();
        p.teleport(getSpawn());
        p.setInstance(this).thenRun(() -> {
            p.setRespawnPoint(getSpawn());
            p.teleport(getSpawn());
            p.setGameMode(GameMode.SURVIVAL);
            p.setAllowFlying(true);

            p.getInventory().setItemStack(3, Items.MORE_SPEED.getItem());
            p.getInventory().setItemStack(4, Items.LESS_SPEED.getItem());

            p.getInventory().setItemStack(7, Items.SPAWN.getItem());
            p.getInventory().setItemStack(8, Items.HUB.getItem());

            p.stopLoading();
        });
    }

    private void writeDefaultConfig(File file) throws IOException {
        FileWriter fileWriter = new FileWriter(file.getPath() + "/config.json");
        String defaultConfig = """
                {
                  "enabled": false,
                  "name": "DEFAULT_NAME",
                  "material": "GRASS_BLOCK",
                  "spawn": {
                    "x": 0,
                    "y": 0,
                    "z": 0
                  },
                  "tags": [
                    "RANDOM_TAG"
                  ]
                }""";
        fileWriter.write(defaultConfig);
        fileWriter.close();
    }

    public ItemStack getItem() {
        return ItemStack.builder(material).displayName(Component.text(name)).build();
    }

    public String getName() {
        return name;
    }

    public Pos getSpawn() {
        return spawn;
    }
    public boolean isEnabled() {
        return enabled;
    }
}
